@@ -13,17 +13,17 @@
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the 
-# Free Software Foundation, Inc., 
+# along with this program; if not, write to the
+# Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
-# 
+#
 #
 
 #set min 3
@@ -69,7 +69,7 @@ proc fasta2frag::dnamut_basechange {dnaseq} {
 	    incr i
 	}
     }
-    
+
     puts "oldseq: $dnaseq"
     set dnaseq [string replace $dnaseq $cpos $cpos $newbase]
     puts "newseq: $dnaseq"
@@ -99,7 +99,7 @@ proc fasta2frag::dnamut {dnaseq min max} {
     for {set i 1} {$i <= $nummut} {incr i} {
 	puts "mut $i"
 	#set dnaseq [dnamut_basechange $dnaseq]
-	
+
 	#set muttype [expr {int(rand()*3)}]
 	#switch -exact -- $muttype {
 	#    0 { set $dnaseq [dnamut_basechange $dnaseq] }
@@ -117,69 +117,13 @@ proc fasta2frag::dumpfasta {name descline dnaseq fout qualseq qout} {
     if {$opts(-minmut) || $opts(-maxmut)} {
 	set dnaseq [dnamut $dnaseq $opts(-minmut) $opts(-maxmut)]
     }
-	
+
     puts $fout ">$name $descline\n$dnaseq"
     if {[llength $qualseq]} {
 	puts $qout ">$name\n$qualseq"
     }
 }
 
-proc fasta2frag::getStrobeData {dnaseq quals from len} {
-    variable opts
-
-    set retseq ""
-    set retquals {}
-
-    set dnalen [string length $dnaseq]
-
-    set thisstrobe $opts(-strobeon)
-    set totalbases 0
-    set ii 0
-
-    for {} {$totalbases < $len} {incr ii} {
-	incr totalbases
-	set dnapos [expr {($from+$ii)}]
-
-	if {$dnapos < $dnalen || $opts(-c)} {
-	    set realindex [expr {$dnapos%$dnalen}]
-	    append retseq [string index $dnaseq $realindex ]
-	    if {[llength $quals]>0} {
-		lappend retquals [lindex $quals $realindex]
-	    } else {
-		lappend retquals $opts(-q)
-	    }
-	    incr thisstrobe -1
-	    if {$thisstrobe ==0 && $totalbases < $len} {
-		if {1} {
-		    # fixed number of 'N', +/- 20% real advance
-		    # this is how I expect PacBio data to be
-		    for {set i 0} {$i < $opts(-strobeoff)} {incr i} {
-			append retseq n
-			lappend retquals 0
-		    }
-		    # variance of +/- 20%
-		    set iiincr [expr {int($opts(-strobeoff)+(rand() * $opts(-strobeoff) * .4)-($opts(-strobeoff)*.2))}]
-		    incr ii $iiincr
-		    set thisstrobe $opts(-strobeon)
-		} else {
-		    # +/- 20% number of 'N', fixed real advance
-		    # for testing only, this should never be a real world case!
-		    
-		    set fto [expr {int($opts(-strobeoff)+(rand() * $opts(-strobeoff) * .4)-($opts(-strobeoff)*.2))}]
-		    for {set i 0} {$i < $fto} {incr i} {
-			append retseq n
-			lappend retquals 0
-		    }
-		    
-		    incr ii $opts(-strobeoff)
-		    set thisstrobe $opts(-strobeon)
-		}
-	    }
-	}
-    }
-
-    return [list $retseq $retquals]
-}
 
 #proc fasta2frag::getdnafromto {dnaseq from to} {
 #    variable opts
@@ -206,7 +150,7 @@ proc fasta2frag::getStrobeData {dnaseq quals from len} {
 
 proc fasta2frag::getqualfromto {dnaseq quals from to} {
     variable opts
-    
+
     set qualslen [llength $quals]
 
     set qualseq {}
@@ -216,12 +160,12 @@ proc fasta2frag::getqualfromto {dnaseq quals from to} {
 	    catch {
 		set thisqual [expr {int([lindex $quals [expr {$i%$qualslen}] ] / $opts(-qualdivisor))}]
 	    }
-	    
+
 	    # noqual?
 	    if {$thisqual < 0} {
 		set thisqual $opts(-minqual)
 	    }
-	    
+
 	    if {$thisqual < $opts(-minqual)} {
 		set thisqual $opts(-minqual)
 	    }
@@ -254,15 +198,15 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
     #close $fid
 
     puts "fragging $name"
-    
+
     #if {[regsub -all -line {^>(.*?)$} $data "" nonamedata] >1} {
     #	puts stderr "More than one sequence in fasta file, aborting."
     #	exit 500
     #}
-    
+
     regsub -all {\n} $nonamedata "" data
     unset nonamedata
-    
+
     # when circularising is on, append first $opts(-l)-1 bases
     #  of sequence to end of sequence (and also quals if present)
     # if strobing, also add $opts(-l)-$opts(-strobeoff) bases
@@ -305,10 +249,13 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 	    set fwdname "TF"
 	    set revname "TR"
 	}
-	default {
+	sanger {
 	    set fwdname "_pe.p1"
 	    set revname "_pe.q1"
 	}
+	default {
+	    error "Unrecognised -pairednaming $opt(-pairednaming)"
+	}
     }
 
     set seqid 1
@@ -330,82 +277,22 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 	return [expr {$from+$opts(-l)}]
     }
 
-    #for { } {$from < $datalen} { incr from $opts(-i); incr to $opts(-i); incr seqid} 
+    #for { } {$from < $datalen} { incr from $opts(-i); incr to $opts(-i); incr seqid}
     for { } {$from < $datalen} { incr from $opts(-i); incr seqid} {
 	#set to [expr {$from+$opts(-l)}]
 	set to [getFragLen $from]
 	if {$lastflag == 0} {
 	    #puts "$from $to"
-	    
+
 	    if {$to >= $datalen} {
 		set lastflag 1
 	    }
-	    
+
 	    if {$opts(-p) > 0 } {
 		set pairnamef ${name}_${seqid}${fwdname}
 		set pairnamer ${name}_${seqid}${revname}
 		if { $opts(-s) > 0 } {
-		    set to2 [expr {$from + $opts(-insert_size)}]
-
-		    if {$to > $datalen} {
-			set to $datalen
-		    }
-		    if {$to2 > $datalen} {
-			set to2 $datalen
-		    }
-
-		    set fractionoff [expr {1.0/$opts(-strobeon)*$opts(-strobeoff)}]
-		    set mult [expr {1.0+$fractionoff}]
-		    set from2 [expr {int($to2 - ($opts(-l)*$mult)) }]
-
-		    if {$to2 <= $datalen} {
-			
-			set rev1 0
-			set rev2 0
-			if { $opts(-r) > 0 } {
-			    if { [expr {$seqid % $opts(-r)}] == 0} {
-				set rev1 1
-				set rev2 1
-			    }
-			}
-			if {$opts(-P)} {
-			    set rev2 [expr {$rev2 ^ 1}]
-			}
-
-			#putz "F: $from $opts(-l)"
-			set bla [getStrobeData $data $quals $from $opts(-l)]
-			set dnaseq [lindex $bla 0]
-			set qualseq [lindex $bla 1]
-			
-			set descline "[expr {$opts(-insert_size)-$opts(-insert_stdev)}] $opts(-insert_size) [expr {$opts(-insert_size)+$opts(-insert_stdev)}] 1 [string length $dnaseq]"
-
-			if {$rev1} {
-			    reverse_seq_qual dnaseq qualseq
-			}
-			dumpfasta $pairnamef$opts(-namesuffix) $descline $dnaseq $fout $qualseq $qout
-			
-			if {$seqid == 1} {
-			    # if we a re working with reversing sequences,
-			    #  force a reverse of the sequence
-			    if { $opts(-r) > 0 } {
-				reverse_seq_qual dnaseq qualseq
-			    }
-			    dumpfasta ${name}$opts(-namesuffix)_${seqid}_a $descline $dnaseq $fout $qualseq $qout
-			}
-
-			#putz "R: $from2 $opts(-l)"
-			set bla [getStrobeData $data $quals $from2 $opts(-l)]
-			set dnaseq [lindex $bla 0]
-			set qualseq [lindex $bla 1]
-			
-			if {$rev2} {
-			    reverse_seq_qual dnaseq qualseq
-			}
-			dumpfasta $pairnamer$opts(-namesuffix) $descline $dnaseq $fout $qualseq $qout
-
-			puts $mpout "$pairnamef$opts(-namesuffix)\t$pairnamer$opts(-namesuffix)"
-			puts $mbout "$pairnamef$opts(-namesuffix)\t$pairnamer$opts(-namesuffix)\tsimplepair"
-		    }
+		    error "forget about that strobe thingy"
 		} else {
 		    set to2 [expr {$from + $opts(-insert_size)}]
 		    set from2 [expr {$to2 - $opts(-l)}]
@@ -419,15 +306,28 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 
 			set rev1 0
 			set rev2 0
-			if { $opts(-r) > 0 } {
-			    if { [expr {$seqid % $opts(-r)}] == 0} {
-				set rev1 1
-				set rev2 1
+			if { [expr {$seqid % 2}] == 0 } {
+			    switch -exact -- $opts(-r) {
+				FF { }
+				RF { set rev1 1}
+				FR { set rev2 1}
+				RR { set rev1 1; set rev2 1 }
+				default { error "-r $opts(-r) is not known, please use FF, RR, FR, RF."}
+			    }
+			} else {
+			    set tmp $pairnamef
+			    set pairnamef $pairnamer
+			    set pairnamer $tmp
+			    switch -exact -- $opts(-r) {
+				FF { set rev1 1; set rev2 1 }
+				RF { set rev1 1}
+				FR { set rev2 1}
+				RR { }
+				default { error "-r $opts(-r) is not known, please use FF, RR, FR, RF."}
 			    }
 			}
-			if {$opts(-P)} {
-			    set rev2 [expr {$rev2 ^ 1}]
-			}
+
+			#putz "$pairnamef $rev1\t$pairnamer $rev2"
 
 			set dnaseq [string range $data $from [expr {$to-1}]]
 			set qualseq [getqualfromto $dnaseq $quals $from $to]
@@ -439,12 +339,13 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 			if {$rev1} {
 			    reverse_seq_qual dnaseq qualseq
 			}
+			#putz "$pairnamef $rev1 $dnaseq"
 			dumpfasta $pairnamef$opts(-namesuffix) $descline $dnaseq $fout $qualseq $qout
-			
+
 			if {$seqid == 1} {
-			    # if we a re working with reversing sequences,
+			    # if we are working with reversing sequences,
 			    #  force a reverse of the sequence
-			    if { $opts(-r) > 0 } {
+			    if { $opts(-r) != "FF" } {
 				reverse_seq_qual dnaseq qualseq
 			    }
 			    dumpfasta ${name}$opts(-namesuffix)_${seqid}_a $descline $dnaseq $fout $qualseq $qout
@@ -455,6 +356,7 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 			if {$rev2} {
 			    reverse_seq_qual dnaseq qualseq
 			}
+			#putz "$pairnamer $rev2 $dnaseq"
 			dumpfasta $pairnamer$opts(-namesuffix) $descline $dnaseq $fout $qualseq $qout
 
 			puts $mpout "$pairnamef\t$pairnamer"
@@ -462,31 +364,25 @@ proc fasta2frag::fragASeq {nonamedata name fout quals qout mpout mbout} {
 		    }
 		}
 	    } else {
-		if { $opts(-s) > 0 } {
-		    set bla [getStrobeData $data $quals $from $opts(-l)]
-		    set dnaseq [lindex $bla 0]
-		    set qualseq [lindex $bla 1]
-		} else {
-		    set dnaseq [string range $data $from [expr {$to-1}]]
-		    set qualseq [getqualfromto $dnaseq $quals $from $to]
-                }
-	
-		if { $opts(-r) > 0 } {
-		    if { [expr {$seqid % $opts(-r)}] == 0} {
+		set dnaseq [string range $data $from [expr {$to-1}]]
+		set qualseq [getqualfromto $dnaseq $quals $from $to]
+
+		if { $opts(-r) != "FF" } {
+		    if { [expr {$seqid % 2}] == 0} {
 			reverse_seq_qual dnaseq qualseq
 		    }
 		}
 
 		set descline "1 [string length $dnaseq]"
 		dumpfasta ${name}$opts(-namesuffix)_$seqid $descline $dnaseq $fout $qualseq $qout
-		
+
 		# the very first and very last fragment will be put twice
 		#  into the data set so that every part of the contig
 		#  is covered at least twice
 		if {$seqid == 1 || $lastflag >0 } {
 		    # if we a re working with reversing sequences,
 		    #  force a reverse of the sequence
-		    if { $opts(-r) > 0 } {
+		    if { $opts(-r) != "FF" } {
 			reverse_seq_qual dnaseq qualseq
 		    }
 		    dumpfasta ${name}$opts(-namesuffix)_${seqid}_a $descline $dnaseq $fout $qualseq $qout
@@ -520,7 +416,7 @@ proc fasta2frag::processit {} {
 
     set fin [open $opts(-infile) r]
     set fout [open $opts(-outfile) w]
-    
+
     set havequal 0
     if {[file exist ${opts(-infile)}.qual]} {
 	puts "have qual"
@@ -560,7 +456,7 @@ proc fasta2frag::processit {} {
 		}
 		set quals [loadNextQuals $qin]
 	    }
-	    
+
 	    fragASeq $actseq $seqname $fout $quals $qout $mpout $mbout
 	    incr cid
 	    set actseq ""
@@ -588,18 +484,30 @@ proc fasta2frag::processit {} {
     }
 }
 
+proc fasta2frag::sanitycheck {} {
+    variable opts
+}
+
+proc fasta2frag::parsequick {qstr} {
+    variable opts
+}
+
 proc usage {prgname} {
     puts stderr "$prgname: Splits a single fasta sequence into several
 overlapping fragments.\n"
     puts stderr "Usage: $prgname ?options? infile outfile
+\t-quick   string      ''
 \t-l   int      Length of fragments (default=3000)
 \t-i   int      Increment of fragment start site (default=2500)
-\t-p   int      Paired end (default=0 is off, 1 is on)
-\t-P   int      In paired-end mode, reverse one of the reads
-\t               (0 is off, default=1 is on)
+\t-p   int      Paired reads (default=0 is off, 1 is on)
+\t-r   string   In shotgun: 'FF' to not to reverse every second read
+\t              In paired reads mode:
+                  FF: forward-forward
+                  RR: reverse-reverse (Ion Torrent, 454)
+                  FR: forward-reverse (Solexa paired-end)
+                  RF: reverse-forward (Solexa mate-pair)
 \t-s   int      Strobe sequencing (default=0 is off, 1 is on)
 \t-q   int      Default quality when no quality data present (default=30)
-\t-r   int      Reverse every nth fragment (default=2)
 \t-c   int      Circularise fragments so that they form a ring
                  (default=0 is is off, 1 would be on)
 
@@ -608,8 +516,8 @@ overlapping fragments.\n"
 \t-insert_size  int      paired-end: insert size (default=3000)
 \t-insert_stdev int      paired-end: standard dev (default=900)
 \t                        this is not working at the moment
-\t-pairednaming string   naming scheme for paired-end: 
-\t                        sanger, 454 (default) or solexa
+\t-pairednaming string   naming scheme for paired-end:
+\t                        sanger, tigr, 454 or solexa (default)
 \t-minmut       int      min. number of mutations/seq. errors (def=0)
 \t-maxmut       int      max. number of mutations/seq. errors (def=0)
 \t-strobeon     int      number of bases read during strobe on
@@ -624,9 +532,8 @@ overlapping fragments.\n"
 
 set fasta2frag::opts(-l) 3000
 set fasta2frag::opts(-i) 2500
-set fasta2frag::opts(-r) 2
+set fasta2frag::opts(-r) FR
 set fasta2frag::opts(-p) 0
-set fasta2frag::opts(-P) 1
 set fasta2frag::opts(-s) 0
 set fasta2frag::opts(-q) 30
 set fasta2frag::opts(-c) 0
@@ -634,7 +541,7 @@ set fasta2frag::opts(-c) 0
 #set fasta2frag::opts(-maxmut) 2
 set fasta2frag::opts(-minmut) 0
 set fasta2frag::opts(-maxmut) 0
-set fasta2frag::opts(-pairednaming) 454
+set fasta2frag::opts(-pairednaming) solexa
 set fasta2frag::opts(-qualdivisor) 1
 set fasta2frag::opts(-minqual) 0
 set fasta2frag::opts(-insert_size) 3000
@@ -650,23 +557,27 @@ set fasta2frag::opts(-namesuffix) ""
 
 foreach {key val} $argv {
   if {![info exists fasta2frag::opts($key)]} {
-      if {[string compare [string index $key 0] "-"] == 0} { 
+      if {[string compare [string index $key 0] "-"] == 0} {
 	  puts stderr "Bad key $key\n"
 	  usage $argv0
       }
       set val $key
       set key -infile
   }
+  if { $key == "-quick" } {
+    fasta2frag::parsequick $val
+  }
   set fasta2frag::opts($key) $val
 }
 
-if {[string length $fasta2frag::opts(-infile)] ==0} { 
+if {[string length $fasta2frag::opts(-infile)] ==0} {
     puts "Missing '-infile' as keyword"
     usage $argv0 ;
 }
-if {[string length $fasta2frag::opts(-outfile)] ==0} { 
+if {[string length $fasta2frag::opts(-outfile)] ==0} {
     puts "Missing '-outfile' as keyword"
-    usage $argv0 ; 
+    usage $argv0 ;
 }
 
+fasta2frag::sanitycheck
 fasta2frag::processit
